@@ -27,7 +27,6 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <X11/Xlib.h>
-#include <gconf/gconf-client.h>
 #include "jawutil.h"
 #include "jawimpl.h"
 #include "jawtoplevel.h"
@@ -54,7 +53,6 @@ gboolean jaw_debug = FALSE;
 
 static gint key_dispatch_result;
 static GMainLoop* jni_main_loop;
-static jobject lock;
 
 static gboolean jaw_initialized = FALSE;
 
@@ -72,30 +70,6 @@ jaw_accessibility_shutdown (void)
   atk_bridge_adaptor_cleanup();
 }
 
-void
-jni_threads_lock()
-{
-  JNIEnv* env;
-
-  env = jaw_util_get_jni_env();
-
-  if ((*env)->MonitorEnter(env, lock) != JNI_OK) {
-    if (jaw_debug)
-      g_critical("Error trying to get Java side GDK lock");
-  }
-}
-void
-jni_threads_unlock()
-{
-  JNIEnv* env;
-  env = jaw_util_get_jni_env();
-
-  if ((*env)->MonitorExit(env, lock) != JNI_OK){
-    if (jaw_debug)
-      g_critical("Error trying to release Java side GDK lock");
-  }
-}
-
 static gpointer jni_loop_callback(void *data)
 {
   if (!g_main_loop_is_running((GMainLoop *)data))
@@ -147,7 +121,6 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_loadAtkBridge(JNIEnv *jniEnv,
   char * message;
   message = "JNI main loop";
   err = NULL;
-  jobject obj;
 
   jaw_initialized = jaw_accessibility_init();
   if (jaw_debug)
@@ -163,9 +136,6 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_loadAtkBridge(JNIEnv *jniEnv,
       g_error_free (err);
     }
   }
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, obj);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
 }
 
 enum _SignalType {
@@ -253,14 +223,14 @@ focus_notify_handler (gpointer p)
     if (jaw_debug)
       g_warning("\nfocus_notify_handler: env == NULL\n");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
   if (global_ac == NULL)
   {
     if (jaw_debug)
       g_warning("\nfocus_notify_handler: global_ac == NULL\n");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
@@ -268,7 +238,7 @@ focus_notify_handler (gpointer p)
     if (jaw_debug)
       g_warning("\nfocus_notify_handler: jaw_impl == NULL\n");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -278,7 +248,7 @@ focus_notify_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -288,7 +258,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_focusNotify(JNIEnv *jniEnv,
 {
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(focus_notify_handler, para);
+  gdk_threads_add_idle(focus_notify_handler, para);
 }
 
 static gboolean
@@ -304,7 +274,7 @@ window_open_handler (gpointer p)
     if (jaw_debug)
       fprintf(stderr,"\n *** window_open_handler: jniEnv == NULL *** \n");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -312,7 +282,7 @@ window_open_handler (gpointer p)
     if (jaw_debug)
       fprintf(stderr,"\n *** window_open_handler: global_ac == NULL *** \n");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
@@ -327,13 +297,13 @@ window_open_handler (gpointer p)
                  "redundant object"))
   {
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (atk_object_get_role(atk_obj) == ATK_ROLE_TOOL_TIP)
   {
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (is_toplevel)
@@ -353,7 +323,7 @@ window_open_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -366,7 +336,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowOpen(JNIEnv *jniEnv,
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
   para->is_toplevel = (jIsToplevel == JNI_TRUE) ? TRUE : FALSE;
-  g_idle_add(window_open_handler, para);
+  gdk_threads_add_idle(window_open_handler, para);
 }
 
 static gboolean
@@ -382,7 +352,7 @@ window_close_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_close_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -390,7 +360,7 @@ window_close_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_close_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
   JawImpl *jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
@@ -398,7 +368,7 @@ window_close_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_close_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -406,13 +376,13 @@ window_close_handler (gpointer p)
   if (!g_strcmp0(atk_role_get_name(atk_object_get_role(atk_obj)), "redundant object"))
   {
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (atk_object_get_role(atk_obj) == ATK_ROLE_TOOL_TIP)
   {
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (is_toplevel) {
@@ -431,7 +401,7 @@ window_close_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -443,7 +413,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowClose(JNIEnv *jniEnv,
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
   para->is_toplevel = (jIsToplevel == JNI_TRUE) ? TRUE : FALSE;
-  g_idle_add(window_close_handler, para);
+  gdk_threads_add_idle(window_close_handler, para);
 }
 
 static gboolean
@@ -458,7 +428,7 @@ window_minimize_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_minimize_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -466,7 +436,7 @@ window_minimize_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_minimize_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
   JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
@@ -474,7 +444,7 @@ window_minimize_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_minimize_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -482,7 +452,7 @@ window_minimize_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -492,7 +462,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowMinimize(JNIEnv *jniEnv,
 {
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(window_minimize_handler, para);
+  gdk_threads_add_idle(window_minimize_handler, para);
 }
 
 static gboolean
@@ -507,7 +477,7 @@ window_maximize_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_maximize_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -515,7 +485,7 @@ window_maximize_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_maximize_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
@@ -524,7 +494,7 @@ window_maximize_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_maximize_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -532,7 +502,7 @@ window_maximize_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowMaximize(JNIEnv *jniEnv,
@@ -541,7 +511,7 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowMaximize(JN
 {
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac );
-  g_idle_add(window_maximize_handler, para);
+  gdk_threads_add_idle(window_maximize_handler, para);
 }
 
 static gboolean
@@ -556,7 +526,7 @@ window_restore_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_restore_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -564,7 +534,7 @@ window_restore_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_restore_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
@@ -573,7 +543,7 @@ window_restore_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_restore_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -581,7 +551,7 @@ window_restore_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowRestore(JNIEnv *jniEnv,
@@ -591,7 +561,7 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowRestore(JNI
 
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(window_restore_handler, para);
+  gdk_threads_add_idle(window_restore_handler, para);
 }
 
 static gboolean
@@ -606,7 +576,7 @@ window_activate_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_activate_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -614,7 +584,7 @@ window_activate_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_activate_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
@@ -622,7 +592,7 @@ window_activate_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_activate_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -630,7 +600,7 @@ window_activate_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowActivate(JNIEnv *jniEnv,
@@ -639,7 +609,7 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowActivate(JN
 
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(window_activate_handler, para);
+  gdk_threads_add_idle(window_activate_handler, para);
 }
 
 static gboolean
@@ -654,7 +624,7 @@ window_deactivate_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_deactivate_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -662,7 +632,7 @@ window_deactivate_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_deactivate_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
@@ -670,7 +640,7 @@ window_deactivate_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_deactivate_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -678,7 +648,7 @@ window_deactivate_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -689,7 +659,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowDeactivate(JNIEnv *jniEnv,
 
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(window_deactivate_handler, para);
+  gdk_threads_add_idle(window_deactivate_handler, para);
 }
 
 static gboolean
@@ -704,7 +674,7 @@ window_state_change_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_state_change_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -712,7 +682,7 @@ window_state_change_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_state_change_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
@@ -721,7 +691,7 @@ window_state_change_handler (gpointer p)
     if (jaw_debug)
       g_warning("window_state_change_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -729,7 +699,7 @@ window_state_change_handler (gpointer p)
 
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -740,7 +710,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowStateChange(JNIEnv *jniEnv
 
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(window_state_change_handler, para);
+  gdk_threads_add_idle(window_state_change_handler, para);
 }
 
 static gchar
@@ -793,7 +763,7 @@ signal_emit_handler (gpointer p)
     if (jaw_debug)
       g_warning("signal_emit_handler: jniEnv == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   jobjectArray args = para->args;
@@ -803,7 +773,7 @@ signal_emit_handler (gpointer p)
     if (jaw_debug)
       g_warning("signal_emit_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
@@ -813,7 +783,7 @@ signal_emit_handler (gpointer p)
     if (jaw_debug)
       g_warning("signal_emit_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -862,7 +832,7 @@ signal_emit_handler (gpointer p)
         if (jaw_debug)
           g_warning("signal_emit_handler: child_impl == NULL");
         free_callback_para(para);
-        return FALSE;
+        return G_SOURCE_REMOVE;
       }
       if (!child_impl)
       {
@@ -903,7 +873,7 @@ signal_emit_handler (gpointer p)
         if (jaw_debug)
           g_warning("signal_emit_handler: child_impl == NULL");
         free_callback_para(para);
-        return FALSE;
+        return G_SOURCE_REMOVE;
       }
       if (!child_impl)
       {
@@ -1062,7 +1032,7 @@ signal_emit_handler (gpointer p)
       break;
   }
   free_callback_para(para);
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -1077,7 +1047,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_emitSignal(JNIEnv *jniEnv,
   CallbackPara *para = alloc_callback_para(global_ac);
   para->signal_id = (gint)id;
   para->args = global_args;
-  g_idle_add(signal_emit_handler, para);
+  gdk_threads_add_idle(signal_emit_handler, para);
 }
 
 static gboolean
@@ -1092,7 +1062,7 @@ object_state_change_handler (gpointer p)
     if (jaw_debug)
       g_warning("object_state_change_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -1100,7 +1070,7 @@ object_state_change_handler (gpointer p)
     if (jaw_debug)
       g_warning("object_state_change_handler: global_ac");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
@@ -1109,7 +1079,7 @@ object_state_change_handler (gpointer p)
     if (jaw_debug)
       g_warning("object_state_change_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   atk_object_notify_state_change(ATK_OBJECT(jaw_impl),
@@ -1117,8 +1087,7 @@ object_state_change_handler (gpointer p)
                                  para->state_value);
 
   free_callback_para(para);
-
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -1137,7 +1106,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_objectStateChange(JNIEnv *jniEnv
   } else {
     para->state_value = FALSE;
   }
-  g_idle_add(object_state_change_handler, para);
+  gdk_threads_add_idle(object_state_change_handler, para);
 }
 
 static gboolean
@@ -1154,7 +1123,7 @@ component_added_handler (gpointer p)
     if (jaw_debug)
       g_warning("component_added_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -1162,7 +1131,7 @@ component_added_handler (gpointer p)
     if (jaw_debug)
       g_warning("component_added_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
@@ -1171,7 +1140,7 @@ component_added_handler (gpointer p)
     if (jaw_debug)
       g_warning("component_added_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
 
@@ -1184,7 +1153,7 @@ component_added_handler (gpointer p)
   }
 
   free_callback_para(para);
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -1194,7 +1163,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_componentAdded(JNIEnv *jniEnv,
 {
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(component_added_handler, para);
+  gdk_threads_add_idle(component_added_handler, para);
 }
 
 static gboolean
@@ -1210,7 +1179,7 @@ component_removed_handler (gpointer p)
     if (jaw_debug)
       g_warning("component_removed_handler: env == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   if (global_ac == NULL)
@@ -1218,7 +1187,7 @@ component_removed_handler (gpointer p)
     if (jaw_debug)
       g_warning("component_removed_handler: global_ac == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
@@ -1227,7 +1196,7 @@ component_removed_handler (gpointer p)
     if (jaw_debug)
       g_warning("component_removed_handler: jaw_impl == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
@@ -1237,13 +1206,13 @@ component_removed_handler (gpointer p)
     if (jaw_debug)
       g_warning("component_removed_handler: atk_obj == NULL");
     free_callback_para(para);
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
   if (atk_object_get_role(atk_obj) == ATK_ROLE_TOOL_TIP)
     atk_object_notify_state_change(atk_obj, ATK_STATE_SHOWING, FALSE);
   free_callback_para(para);
 
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT void
@@ -1253,7 +1222,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_componentRemoved(JNIEnv *jniEnv,
 {
   jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   CallbackPara *para = alloc_callback_para(global_ac);
-  g_idle_add(component_removed_handler, para);
+  gdk_threads_add_idle(component_removed_handler, para);
 }
 
 static gboolean
@@ -1268,7 +1237,7 @@ key_dispatch_handler (gpointer p)
   {
     if (jaw_debug)
       g_warning("key_dispatch_handler: env == NULL");
-    return FALSE;
+    return G_SOURCE_REMOVE;
   }
 
   jclass classAtkKeyEvent = (*jniEnv)->FindClass(jniEnv, "org/GNOME/Accessibility/AtkKeyEvent");
@@ -1361,7 +1330,7 @@ key_dispatch_handler (gpointer p)
   g_free(event);
 
   (*jniEnv)->DeleteGlobalRef(jniEnv, jAtkKeyEvent);
-  return FALSE;
+  return G_SOURCE_REMOVE;
 }
 
 JNIEXPORT jboolean
@@ -1371,7 +1340,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_dispatchKeyEvent(JNIEnv *jniEnv,
 {
   jboolean key_consumed;
   jobject global_key_event = (*jniEnv)->NewGlobalRef(jniEnv, jAtkKeyEvent);
-  g_idle_add(key_dispatch_handler, (gpointer)global_key_event);
+  gdk_threads_add_idle(key_dispatch_handler, (gpointer)global_key_event);
 
   if(jaw_debug)
     printf("key_dispatch_result saved = %d\n ", key_dispatch_result);
