@@ -65,6 +65,10 @@ static gboolean			jaw_table_is_row_selected		(AtkTable	*table,
 static gboolean			jaw_table_is_selected			(AtkTable	*table,
 									 gint		row,
 									 gint		column);
+static gboolean jaw_table_add_row_selection(AtkTable  *table,
+                                            gint       row);
+static gboolean jaw_table_add_column_selection(AtkTable  *table,
+                                               gint      column);
 
 typedef struct _TableData {
 	jobject atk_table;
@@ -94,6 +98,8 @@ jaw_table_interface_init (AtkTableIface *iface)
 	iface->is_column_selected = jaw_table_is_column_selected;
 	iface->is_row_selected = jaw_table_is_row_selected;
 	iface->is_selected = jaw_table_is_selected;
+  iface->add_row_selection= jaw_table_add_row_selection;
+  iface->add_column_selection= jaw_table_add_column_selection;
 }
 
 gpointer
@@ -147,7 +153,8 @@ jaw_table_ref_at (AtkTable *table, gint	row, gint column)
 
 	JawImpl* jaw_impl = jaw_impl_get_instance( jniEnv, jac );
 
-	g_object_ref( G_OBJECT(jaw_impl) );
+  if (G_OBJECT(jaw_impl) != NULL)
+    g_object_ref(G_OBJECT(jaw_impl));
 
 	return ATK_OBJECT(jaw_impl);
 }
@@ -504,3 +511,38 @@ jaw_table_is_selected (AtkTable *table, gint row, gint column)
 	return FALSE;
 }
 
+static gboolean
+jaw_table_add_row_selection(AtkTable *table, gint row)
+{
+  JawObject *jaw_obj = JAW_OBJECT(table);
+  TableData *data = jaw_object_get_interface_data(jaw_obj, INTERFACE_TABLE);
+  jobject atk_table = data->atk_table;
+
+  JNIEnv *jniEnv = jaw_util_get_jni_env();
+  jclass classAtkTable = (*jniEnv)->FindClass(jniEnv, "org/GNOME/Accessibility/AtkTable");
+  jmethodID jmid = (*jniEnv)->GetMethodID(jniEnv, classAtkTable, "addRowSelection", "(I)Z");
+  jboolean jselected = (*jniEnv)->CallBooleanMethod(jniEnv, atk_table, jmid, (jint)row);
+
+  if (jselected == JNI_TRUE)
+    return TRUE;
+
+  return FALSE;
+}
+
+static gboolean
+jaw_table_add_column_selection(AtkTable *table, gint column)
+{
+  JawObject *jaw_obj = JAW_OBJECT(table);
+  TableData *data = jaw_object_get_interface_data(jaw_obj, INTERFACE_TABLE);
+  jobject atk_table = data->atk_table;
+
+  JNIEnv *jniEnv = jaw_util_get_jni_env();
+  jclass classAtkTable = (*jniEnv)->FindClass(jniEnv, "org/GNOME/Accessibility/AtkTable");
+  jmethodID jmid = (*jniEnv)->GetMethodID(jniEnv, classAtkTable, "addColumnSelection", "(I)Z");
+  jboolean jselected = (*jniEnv)->CallBooleanMethod(jniEnv, atk_table, jmid, (jint)column);
+
+  if (jselected == JNI_TRUE)
+    return TRUE;
+
+  return FALSE;
+}
