@@ -31,7 +31,7 @@ public class AtkWrapper {
   static boolean accessibilityEnabled = false;
   static {
     try {
-      Process p = Runtime.getRuntime().exec("/bin/xprop -root");
+      Process p = Runtime.getRuntime().exec("/usr/bin/xprop -root");
       BufferedReader b = new BufferedReader(new InputStreamReader (p.getInputStream ()));
       String result;
       while ((result = b.readLine()) != null) {
@@ -449,7 +449,6 @@ public class AtkWrapper {
         } else if( propertyName.equals(AccessibleContext.ACCESSIBLE_SELECTION_PROPERTY) ) {
           boolean isTextEvent = false;
           AccessibleRole role = ac.getAccessibleRole();
-          System.out.println(role);
           if ((role == AccessibleRole.TEXT) ||
               role.toDisplayString(java.util.Locale.US).equalsIgnoreCase("paragraph")) {
             isTextEvent = true;
